import { Result, createSuccess, createError } from '@/types/result'
import { cloudApi } from '@/api'
import { USE_MOCK } from '@/services'
import {
  type TurtleSoupGameHostView,
  type TurtleSoupGamePlayerView,
  type TurtleSoupGameSettings,
  type TurtleSoupTheme,
  type TurtleSoupPuzzle,
  TurtleSoupGameStatus,
  TurtleSoupDifficulty,
  TurtleSoupThemeType
} from '@/types/games/turtlesoup'

// Mock 主题数据（字段同步 themeId）
const mockThemes: TurtleSoupTheme[] = [
  {
    themeId: 'theme-1',
    name: '经典解谜',
    description: '基础主题 · 免费',
    type: TurtleSoupThemeType.BASIC,
    isLocked: false
  },
  {
    themeId: 'theme-2',
    name: '环球影城',
    description: '特色主题 · 20元/小时',
    type: TurtleSoupThemeType.PREMIUM,
    price: 20,
    isNew: true,
    isLocked: true
  }
]

// Mock 题目数据（字段同步 puzzleId 等）
const mockPuzzles: TurtleSoupPuzzle[] = [
  {
    puzzleId: 'puzzle-1',
    themeId: 'theme-1',
    title: '神秘的手表',
    scenario: '一个男人收到了一块神秘的手表...',
    truth: '这块手表可以预知未来...',
    keyClues: ['这块手表有特殊功能。', '它能预测未来。'],
    hints: [
      { keyword: '特殊功能', sentence: '手表可以预知未来。' }
    ],
    difficulty: TurtleSoupDifficulty.EASY,
    averageDuration: 25
  }
]

// Mock 主持人视图
const mockHostView: TurtleSoupGameHostView = {
  gameId: 'game-1',
  roomId: 'room-1',
  title: '欢乐海龟汤',
  theme: mockThemes[0],
  puzzle: mockPuzzles[0],
  status: TurtleSoupGameStatus.ACTIVE,
  startTime: Date.now() - 1800000,
  currentTime: Date.now(),
  duration: 30,
  hostId: 'host-user',
  hostName: '小明 (我)',
  players: [
    {
      userId: 'user-1',
      name: '小红',
      avatar: '',
      joinedAt: Date.now() - 3000000,
      questionCount: 2
    }
  ],
  progress: 40,
  difficulty: TurtleSoupDifficulty.MEDIUM,
  settings: {
    themeId: 'theme-1',
    puzzleId: 'puzzle-1',
    difficulty: TurtleSoupDifficulty.MEDIUM,
    maxPlayers: 10
  }
}

// Mock 玩家视图
const mockPlayerView: TurtleSoupGamePlayerView = {
  gameId: 'game-1',
  roomId: 'room-1',
  title: '欢乐海龟汤',
  theme: mockThemes[0],
  puzzle: mockPuzzles[0],
  status: TurtleSoupGameStatus.ACTIVE,
  startTime: Date.now() - 1800000,
  currentTime: Date.now(),
  duration: 30,
  hostId: 'host-user',
  hostName: '小明 (我)',
  progress: 40,
  difficulty: TurtleSoupDifficulty.MEDIUM,
  settings: {
    themeId: 'theme-1',
    puzzleId: 'puzzle-1',
    difficulty: TurtleSoupDifficulty.MEDIUM,
    maxPlayers: 10
  },
  myPlayerInfo: {
    userId: 'user-1',
    name: '小红',
    avatar: '',
    joinedAt: Date.now() - 3000000,
    questionCount: 2
  },
  otherPlayers: [
    {
      userId: 'user-2',
      name: '小李',
      avatar: ''
    }
  ]
}

export const turtleSoupService = {
  async getThemes(): Promise<Result<TurtleSoupTheme[]>> {
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => resolve(createSuccess(mockThemes)), 300))
    }
    return cloudApi.call<{ themes: TurtleSoupTheme[] }>('getTurtleSoupThemes')
      .then(result => {
        if (result.success && result.data?.themes) return createSuccess(result.data.themes)
        return createError(result.message || '获取主题列表失败')
      })
      .catch(error => createError(error.message || '获取主题列表失败'))
  },

  async unlockTheme(themeId: string): Promise<Result<TurtleSoupTheme>> {
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => {
        const theme = mockThemes.find(t => t.themeId === themeId)
        resolve(theme ? createSuccess(theme) : createError('主题未找到'))
      }, 300))
    }
    return cloudApi.call<{ theme: TurtleSoupTheme }>('unlockTurtleSoupTheme', { themeId })
      .then(result => result.success && result.data?.theme
        ? createSuccess(result.data.theme)
        : createError(result.message || '解锁主题失败'))
      .catch(error => createError(error.message || '解锁主题失败'))
  },

  async getPuzzles(themeId: string, difficulty?: TurtleSoupDifficulty): Promise<Result<TurtleSoupPuzzle[]>> {
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => {
        let puzzles = mockPuzzles.filter(p => p.themeId === themeId)
        if (difficulty) puzzles = puzzles.filter(p => p.difficulty === difficulty)
        resolve(createSuccess(puzzles))
      }, 300))
    }
    return cloudApi.call<{ puzzles: TurtleSoupPuzzle[] }>('getTurtleSoupPuzzles', { themeId, difficulty })
      .then(result => {
        if (result.success && result.data?.puzzles) return createSuccess(result.data.puzzles)
        return createError(result.message || '获取题目列表失败')
      })
      .catch(error => createError(error.message || '获取题目列表失败'))
  },

  async createGame(settings: TurtleSoupGameSettings): Promise<Result<{ gameId: string, roomId: string, title: string }>> {
    if (!settings.themeId) return createError('请选择游戏主题')
    if (!settings.puzzleId) return createError('请选择游戏题目')
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => {
        const gameId = `game-${Date.now()}`
        const roomId = `room-${Date.now()}`
        resolve(createSuccess({
          gameId,
          roomId,
          title: mockPuzzles.find(p => p.puzzleId === settings.puzzleId)?.title || '新游戏'
        }))
      }, 500))
    }
    return cloudApi.call<{ gameId: string, roomId: string, title: string }>('createTurtleSoupGame', settings)
  },

  async getHostGameData(gameId: string): Promise<Result<TurtleSoupGameHostView>> {
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => resolve(createSuccess(mockHostView)), 300))
    }
    return cloudApi.call<{ game: TurtleSoupGameHostView }>('getTurtleSoupHostGame', { gameId })
      .then(result => result.success && result.data?.game
        ? createSuccess(result.data.game)
        : createError(result.message || '获取游戏数据失败'))
      .catch(error => createError(error.message || '获取主持人游戏数据失败'))
  },

  async getPlayerGameData(gameId: string): Promise<Result<TurtleSoupGamePlayerView>> {
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => resolve(createSuccess(mockPlayerView)), 300))
    }
    return cloudApi.call<{ game: TurtleSoupGamePlayerView }>('getTurtleSoupPlayerGame', { gameId })
      .then(result => result.success && result.data?.game
        ? createSuccess(result.data.game)
        : createError(result.message || '获取游戏数据失败'))
      .catch(error => createError(error.message || '获取玩家游戏数据失败'))
  },

  async startGame(gameId: string): Promise<Result<{ success: boolean }>> {
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => resolve(createSuccess({ success: true })), 300))
    }
    return cloudApi.call<{ success: boolean }>('startTurtleSoupGame', { gameId })
  },

  async submitSolution(gameId: string, solution: string): Promise<Result<{ correct: boolean }>> {
    if (!solution.trim()) return createError('解答内容不能为空')
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => {
        const correct = Math.random() > 0.5
        resolve(createSuccess({ correct }))
      }, 300))
    }
    return cloudApi.call<{ correct: boolean }>('submitTurtleSoupSolution', { gameId, solution })
  },

  async updateProgress(gameId: string, progress: number): Promise<Result<{ success: boolean }>> {
    if (progress < 0 || progress > 100) return createError('进度值必须在0-100之间')
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => resolve(createSuccess({ success: true })), 200))
    }
    return cloudApi.call<{ success: boolean }>('updateTurtleSoupProgress', { gameId, progress })
  },

  async checkGameRole(gameId: string): Promise<Result<{ isHost: boolean, revealedClues: number[] }>> {
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => {
        resolve(createSuccess({ 
          isHost: Math.random() > 0.5, 
          revealedClues: [0, 1]  
        }))
      }, 300))
    }
    return cloudApi.call<{ isHost: boolean, revealedClues: number[] }>('checkTurtleSoupRole', { gameId })
      .then(result => {
        if (result.success && result.data) {
          return createSuccess(result.data)
        }
        return createError(result.message || '检查游戏角色失败')
      })
      .catch(error => createError(error.message || '检查游戏角色失败'))
  },

  async revealClue(gameId: string, clueIndex: number): Promise<Result<{ success: boolean }>> {
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => resolve(createSuccess({ success: true })), 200))
    }
    return cloudApi.call<{ success: boolean }>('revealTurtleSoupClue', { gameId, clueIndex })
  },

  async updateGameSettings(gameId: string, settings: TurtleSoupGameSettings): Promise<Result<{ success: boolean }>> {
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      return new Promise(resolve => setTimeout(() => resolve(createSuccess({ success: true })), 200))
    }
    return cloudApi.call<{ success: boolean }>('updateTurtleSoupSettings', { gameId, settings })
  },

  async getThemePaymentParams(themeId: string): Promise<Result<any>> {
    if (!themeId) return createError('主题ID不能为空')
    
    if (USE_MOCK || process.env.TARO_ENV !== 'weapp') {
      // Mock支付参数
      return new Promise(resolve => setTimeout(() => {
        resolve(createSuccess({
          timeStamp: `${Date.now()}`,
          nonceStr: 'mock_nonce_str',
          package: 'prepay_id=mock_prepay_id',
          signType: 'MD5',
          paySign: 'mock_pay_sign'
        }))
      }, 300))
    }
    
    return cloudApi.call<{
      timeStamp: string,
      nonceStr: string,
      package: string,
      signType: string,
      paySign: string
    }>('getThemePaymentParams', { themeId })
      .then(result => {
        if (result.success && result.data) {
          return createSuccess(result.data)
        }
        return createError(result.message || '获取支付参数失败')
      })
      .catch(error => createError(error.message || '获取支付参数失败'))
  }
}